require "test/unit"
require "TagTreeScanner"

class Tag_Test < Test::Unit::TestCase
	def setup
	end
	
	def test1_tags
		root = TagTreeScanner::Tag.new( :root, { :is_root => true } )
		assert_equal( :root, root.tag_name )
		assert_equal( true, root.attributes[ :is_root ] )
		assert_nil( root.allowed_genre )
		assert( root.allows_text? )

		t1 = TagTreeScanner::Tag.new( :t1 )
		root.append_child( t1 )
		assert_equal( 1, root.child_tags.length )
		assert_equal( t1, root.child_tags.first )

		t2 = TagTreeScanner::Tag.new( :t2 )
		root.append_child( t2 )
		assert_equal( 2, root.child_tags.length )
		assert_equal( t2, root.child_tags.last )

		t3 = TagTreeScanner::Tag.new( :t3 )
		root.insert_before( t3, t2 )
		assert_equal( 3, root.child_tags.length )
		assert_equal( [t1,t3,t2], root.child_tags )

		root.append_child( t1 )
		assert_equal( [t3,t2,t1], root.child_tags )

		t1.replace_with( t3 )
		assert_equal( [t2,t3], root.child_tags )
		assert_nil( t1.parent_tag )

		root.insert_before( t1, t2 )
		assert_equal( [t1,t2,t3], root.child_tags )
		assert_equal( root, t1.parent_tag )

		root.append_child( t1 )
		assert_equal( [t2,t3,t1], root.child_tags )
		assert_equal( root, t1.parent_tag )
		assert_nil( t1.next_sibling )
		assert_nil( t2.previous_sibling )
		
		t1.append_child( t3 )
		assert_equal( [t2,t1], root.child_tags )
		assert_nil( t3.next_sibling )
		assert_nil( t3.previous_sibling )
		assert_equal( t1, t2.next_sibling )
		assert_equal( t2, t1.previous_sibling )
		assert_equal( t3, t1.child_tags.first )
		
		assert_raise( RuntimeError ){
			t3.append_child( t1 )
		}

		assert_raise( RuntimeError ){
			t1.append_child( t1 )
		}
	end

	def test2_tags2
		root = TagTreeScanner::Tag.new( :root )
		# make a ton of tags...
		1.upto(100){ |i|
			root.append_child( TagTreeScanner::Tag.new( "t#{i}".intern ) )
		}
		
		# ...shuffle the hell out of them...
		500.times{ 
			next unless n1 = root.child_tags[ rand( root.child_tags.length ) ]
			n2 = root.child_tags[ rand( root.child_tags.length ) ]
			next if n1 == n2
			case rand(30)
				when 0
					root.remove_child( n1 )
				when 1
					root.append_child( n1 )
				when 2
					root.insert_before( n1, nil )
				when 3
					root.insert_after( n1, nil )
				when 4
					root.insert_before( n1, n2 )
				when 5
					n1.replace_with( n2 )
				else
					root.insert_after( n1, n2 )
			end
		}
		
		# ...and now ensure that they're all properly linked
		last_tag = nil
		root.child_tags.each{ |tag|
			assert_equal( last_tag, tag.previous_sibling )
			assert_equal( tag, last_tag.next_sibling ) if last_tag
			assert_equal( root, tag.parent_tag )
			last_tag = tag
		}
		assert_nil( last_tag.next_sibling ) if last_tag
	end
end